/**
 * @file
 * VuoList_VuoArtNetInputDevice C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoArtNetInputDevice_H
#define VuoList_VuoArtNetInputDevice_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoArtNetInputDevice VuoList_VuoArtNetInputDevice
 * A list of @ref VuoArtNetInputDevice elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoArtNetInputDevice_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoArtNetInputDevice elements.
 */
typedef const struct VuoList_VuoArtNetInputDevice_struct { void *l; } * VuoList_VuoArtNetInputDevice;
/// @}
#endif

/**
 * Creates a new list of @ref VuoArtNetInputDevice elements.
 */
VuoList_VuoArtNetInputDevice VuoListCreate_VuoArtNetInputDevice(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoArtNetInputDevice VuoListCopy_VuoArtNetInputDevice(const VuoList_VuoArtNetInputDevice list);

/**
 * Returns the @ref VuoArtNetInputDevice at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoArtNetInputDevice VuoListGetValue_VuoArtNetInputDevice(const VuoList_VuoArtNetInputDevice list, const unsigned long index);

/**
 * Changes the @ref VuoArtNetInputDevice at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoArtNetInputDevice(const VuoList_VuoArtNetInputDevice list, const VuoArtNetInputDevice value, const unsigned long index);

/**
 * Inserts the `VuoArtNetInputDevice` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoArtNetInputDevice(const VuoList_VuoArtNetInputDevice list, const VuoArtNetInputDevice value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoArtNetInputDevice(VuoList_VuoArtNetInputDevice list, const VuoArtNetInputDevice value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoArtNetInputDevice(VuoList_VuoArtNetInputDevice list, const VuoArtNetInputDevice value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoArtNetInputDevice(VuoList_VuoArtNetInputDevice list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoArtNetInputDevice(VuoList_VuoArtNetInputDevice list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoArtNetInputDevice(VuoList_VuoArtNetInputDevice list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoArtNetInputDevice(VuoList_VuoArtNetInputDevice list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoArtNetInputDevice(VuoList_VuoArtNetInputDevice list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoArtNetInputDevice(VuoList_VuoArtNetInputDevice list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoArtNetInputDevice(VuoList_VuoArtNetInputDevice list);

/**
 * Removes the `VuoArtNetInputDevice` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoArtNetInputDevice(VuoList_VuoArtNetInputDevice list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoArtNetInputDevice(const VuoList_VuoArtNetInputDevice list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoArtNetInputDevice VuoList_VuoArtNetInputDevice_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoArtNetInputDevice_getJson(const VuoList_VuoArtNetInputDevice value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoArtNetInputDevice_getSummary(const VuoList_VuoArtNetInputDevice value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoArtNetInputDevice VuoList_VuoArtNetInputDevice_makeFromString(const char *str);
char * VuoList_VuoArtNetInputDevice_getString(const VuoList_VuoArtNetInputDevice value);
/// @}

/**
 * @}
 */

#endif
