/**
 * @file
 * VuoList_VuoCurve C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoCurve_H
#define VuoList_VuoCurve_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoCurve VuoList_VuoCurve
 * A list of @ref VuoCurve elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoCurve_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoCurve elements.
 */
typedef const struct VuoList_VuoCurve_struct { void *l; } * VuoList_VuoCurve;
/// @}
#endif

/**
 * Creates a new list of @ref VuoCurve elements.
 */
VuoList_VuoCurve VuoListCreate_VuoCurve(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoCurve VuoListCopy_VuoCurve(const VuoList_VuoCurve list);

/**
 * Returns the @ref VuoCurve at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoCurve VuoListGetValue_VuoCurve(const VuoList_VuoCurve list, const unsigned long index);

/**
 * Changes the @ref VuoCurve at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoCurve(const VuoList_VuoCurve list, const VuoCurve value, const unsigned long index);

/**
 * Inserts the `VuoCurve` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoCurve(const VuoList_VuoCurve list, const VuoCurve value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoCurve(VuoList_VuoCurve list, const VuoCurve value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoCurve(VuoList_VuoCurve list, const VuoCurve value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoCurve(VuoList_VuoCurve list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoCurve(VuoList_VuoCurve list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoCurve(VuoList_VuoCurve list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoCurve(VuoList_VuoCurve list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoCurve(VuoList_VuoCurve list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoCurve(VuoList_VuoCurve list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoCurve(VuoList_VuoCurve list);

/**
 * Removes the `VuoCurve` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoCurve(VuoList_VuoCurve list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoCurve(const VuoList_VuoCurve list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoCurve VuoList_VuoCurve_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoCurve_getJson(const VuoList_VuoCurve value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoCurve_getSummary(const VuoList_VuoCurve value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoCurve VuoList_VuoCurve_makeFromString(const char *str);
char * VuoList_VuoCurve_getString(const VuoList_VuoCurve value);
/// @}

/**
 * @}
 */

#endif
