/**
 * @file
 * VuoList_VuoThresholdType C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoThresholdType_H
#define VuoList_VuoThresholdType_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoThresholdType VuoList_VuoThresholdType
 * A list of @ref VuoThresholdType elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoThresholdType_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoThresholdType elements.
 */
typedef const struct VuoList_VuoThresholdType_struct { void *l; } * VuoList_VuoThresholdType;
/// @}
#endif

/**
 * Creates a new list of @ref VuoThresholdType elements.
 */
VuoList_VuoThresholdType VuoListCreate_VuoThresholdType(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoThresholdType VuoListCopy_VuoThresholdType(const VuoList_VuoThresholdType list);

/**
 * Returns the @ref VuoThresholdType at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoThresholdType VuoListGetValue_VuoThresholdType(const VuoList_VuoThresholdType list, const unsigned long index);

/**
 * Changes the @ref VuoThresholdType at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoThresholdType(const VuoList_VuoThresholdType list, const VuoThresholdType value, const unsigned long index);

/**
 * Inserts the `VuoThresholdType` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoThresholdType(const VuoList_VuoThresholdType list, const VuoThresholdType value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoThresholdType(VuoList_VuoThresholdType list, const VuoThresholdType value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoThresholdType(VuoList_VuoThresholdType list, const VuoThresholdType value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoThresholdType(VuoList_VuoThresholdType list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoThresholdType(VuoList_VuoThresholdType list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoThresholdType(VuoList_VuoThresholdType list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoThresholdType(VuoList_VuoThresholdType list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoThresholdType(VuoList_VuoThresholdType list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoThresholdType(VuoList_VuoThresholdType list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoThresholdType(VuoList_VuoThresholdType list);

/**
 * Removes the `VuoThresholdType` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoThresholdType(VuoList_VuoThresholdType list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoThresholdType(const VuoList_VuoThresholdType list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoThresholdType VuoList_VuoThresholdType_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoThresholdType_getJson(const VuoList_VuoThresholdType value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoThresholdType_getSummary(const VuoList_VuoThresholdType value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoThresholdType VuoList_VuoThresholdType_makeFromString(const char *str);
char * VuoList_VuoThresholdType_getString(const VuoList_VuoThresholdType value);
/// @}

/**
 * @}
 */

#endif
