/**
 * @file
 * VuoList_VuoWindowReference C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoWindowReference_H
#define VuoList_VuoWindowReference_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoWindowReference VuoList_VuoWindowReference
 * A list of @ref VuoWindowReference elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoWindowReference_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoWindowReference elements.
 */
typedef const struct VuoList_VuoWindowReference_struct { void *l; } * VuoList_VuoWindowReference;
/// @}
#endif

/**
 * Creates a new list of @ref VuoWindowReference elements.
 */
VuoList_VuoWindowReference VuoListCreate_VuoWindowReference(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoWindowReference VuoListCopy_VuoWindowReference(const VuoList_VuoWindowReference list);

/**
 * Returns the @ref VuoWindowReference at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoWindowReference VuoListGetValue_VuoWindowReference(const VuoList_VuoWindowReference list, const unsigned long index);

/**
 * Changes the @ref VuoWindowReference at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoWindowReference(const VuoList_VuoWindowReference list, const VuoWindowReference value, const unsigned long index);

/**
 * Inserts the `VuoWindowReference` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoWindowReference(const VuoList_VuoWindowReference list, const VuoWindowReference value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoWindowReference(VuoList_VuoWindowReference list, const VuoWindowReference value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoWindowReference(VuoList_VuoWindowReference list, const VuoWindowReference value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoWindowReference(VuoList_VuoWindowReference list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoWindowReference(VuoList_VuoWindowReference list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoWindowReference(VuoList_VuoWindowReference list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoWindowReference(VuoList_VuoWindowReference list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoWindowReference(VuoList_VuoWindowReference list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoWindowReference(VuoList_VuoWindowReference list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoWindowReference(VuoList_VuoWindowReference list);

/**
 * Removes the `VuoWindowReference` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoWindowReference(VuoList_VuoWindowReference list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoWindowReference(const VuoList_VuoWindowReference list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoWindowReference VuoList_VuoWindowReference_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoWindowReference_getJson(const VuoList_VuoWindowReference value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoWindowReference_getSummary(const VuoList_VuoWindowReference value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoWindowReference VuoList_VuoWindowReference_makeFromString(const char *str);
char * VuoList_VuoWindowReference_getString(const VuoList_VuoWindowReference value);
/// @}

/**
 * @}
 */

#endif
